; Fichero LEECAR_I.ASM
; Runtina que lee del teclado por interrupciones para poder mover la
; pala cuando se pulsa 'O' o 'P', o acabar el juego cuando se pulsa 'F'.

TITLE  LEECAR_I

; Rutinas NEAR que utiliza LEECAR_I:
; HUECO_DR, HUECO_IZ.

; Rutinas FAR que utiliza LEECAR_I:  
EXTRN ESCR_CAR:FAR    ;Rutina que escribe un carcter por pantalla.
EXTRN DIB_PALA:FAR    ;Rutina que dibuja la raqueta en la posicin (i,j).

; Variables globales que utiliza LEECAR_I:
EXTRN TABLA:BYTE      ;Tabla que codifica los caracteres del teclado.
EXTRN FIN_JUEGO:BYTE  ;Cuando es una 'F' (33) acaba el juego.
EXTRN POSI_PALA:BYTE  ;Contiene la actual posicin de la pelota.


CODIGO SEGMENT PUBLIC 'CODIGO'
       ASSUME CS:CODIGO

PUBLIC   LEECAR_I
LEECAR_I PROC FAR

       CLI                ;Inhibir interrupciones.
       PUSH AX
       PUSH BX
       PUSH DX            ;Salvar registros.

       IN   AL,60H        ;Leer el cdigo de rastreo.
       XOR  AH,AH         ;Poner a cero el byte alto.

       TEST AL,80H
       JNZ  FIN           ;Si es break se ignora.

       LEA  BX,TABLA      ;Si es MAKE se realiza la traduccin.
       CMP  AL,33
       JZ   LET_F         ;Esto quiere decir que he pulsado la 'F'.

       CMP  AL,25
       JZ   MOV_DR        ;Esto quiere decir que he pulsado la 'P'.

       CMP  AL,24
       JNZ  FIN           ;Sino, entonces he pulsado la 'O'.

; Muevo la raqueta a la izquierda.
MOV_IZ:MOV  BL,POSI_PALA
       CMP  BL,4
       JLE  FIN           ;Si est en el extremo izquierdo, no sigo.
       SUB  BL,1
       MOV  POSI_PALA,BL  ;Sino decrementamos la columna.
       CALL DIB_PALA      ;Subrutina que dibuja la raqueta.

       CALL HUECO_DR      ;Subrutina que borra el extremo derecho de
       JMP  FIN           ;la raqueta.

; Muevo la raqueta a la derecha
MOV_DR:MOV  BL,POSI_PALA
       CMP  BL,64
       JG  FIN           ;Si est en el extremo derecho, no sigo.
       ADD  BX,1
       MOV  POSI_PALA,BL ;Sino incrementamos la columna.
       CALL DIB_PALA     ;Subrutina que dibuja la raqueta.

       CALL HUECO_IZ      ;Subrutina que borra el extremo izquierdo de
       JMP  FIN           ;la raqueta.

; Actualizo la variable FIN_JUEGO.
LET_F: MOV  FIN_JUEGO,AL

FIN:   IN   AL,61H        ;Comienza la secuencia de strobe.
       OR   AL,80H
       OUT  61H,AL
       AND  AL,7FH
       OUT  61H,AL        ;Acaba la secuencia de strobe.

EOI:   MOV  AL,20H        ;EOI: fin de interrupcin para el i8259.
       OUT  20H,AL

       POP  DX
       POP  BX
       POP  AX            ;Restaurar registros.
       STI                ;Habilitar interrupciones.
       IRET               ;Retorno de rutina de servicio.

LEECAR_I  ENDP


; Runtina que borra los caracteres que deja la raquta cuando se mueve
; a la izquierda.
HUECO_DR PROC NEAR
       PUSH BP
       MOV  BP,SP         ;Enlace dinmico.
       PUSH AX
       PUSH SI            ;Salvar registros.

       MOV  SI,19         ;Inicializo la fila.
WHDR2: CMP  SI,21
       JE   FIN_DRC

       MOV  AX,07H
       PUSH AX            ;Empilo el atributo.
       MOV  AX,20H
       PUSH AX            ;Empilo un blanco ' '.

       MOV  AL,POSI_PALA
       ADD  AL,11
       PUSH AX            ;Empilo la columna.
       PUSH SI            ;Empilo la fila.
       CALL ESCR_CAR      ;Llamada a subrutina de escritura.
       INC  SI            ;Incrementar fila.
       JMP  WHDR2

FIN_DRC:POP  SI
       POP   AX           ;Restaurar registros.
       POP   BP           ;Restauramos emlace dinmico.
       RET
HUECO_DR  ENDP

; Runtina que borra los caracteres que deja la pala cuando se mueve
; a la derecha.
HUECO_IZ PROC NEAR

       PUSH BP
       MOV  BP,SP         ;Enlace dinmico.
       PUSH AX
       PUSH SI            ;Salvar registros.

       MOV  SI,19         ;Inicializo la fila.
WHIZ2: CMP  SI,21
       JG   FIN_IZC

       MOV  AX,07H
       PUSH AX            ;Empilo el atributo.
       MOV  AX,20H
       PUSH AX            ;Empilo un blanco ' '.

       MOV  AL,POSI_PALA
       SUB  AL,1
       PUSH AX            ;Empilo la columna.
       PUSH SI            ;Empilo la fila.
       CALL ESCR_CAR      ;llamada a subrutina de escritura.
       INC  SI            ;Incrementar fila.
       JMP  WHIZ2

FIN_IZC:POP  SI
       POP   AX             ;Recuperar registros.
       POP   BP             ;Restaurar emlace dinmico.
       RET
HUECO_IZ  ENDP

CODIGO    ENDS
          END
